@@ -109,6 +109,7 @@ struct GpmManagerPrivate
 
 	gboolean	 use_time_to_notify;
 	gboolean	 lid_is_closed;
+	gboolean	 done_notify_fully_charged;
 
 	time_t           last_resume_event;
 	int		 suppress_policy_timeout;
@@ -1215,7 +1216,7 @@ power_on_ac_changed_cb (GpmPower   *power,
 
 	/* We do the lid close on battery action if the ac_adapter is removed
 	   when the laptop is closed and on battery. Fixes #331655 */
-	if (! on_ac && manager->priv->lid_is_closed) {
+	if ( (! on_ac) && manager->priv->lid_is_closed) {
 		gpm_debug ("Doing battery policy when lid closed and power removed");
 		gpm_manager_set_reason (manager, "the lid has been closed, and the ac adapter removed");
 		manager_policy_do (manager, GPM_PREF_BATTERY_BUTTON_LID);
@@ -1319,7 +1320,7 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 	/* If we are charging we should show warnings again as soon as we discharge again */
 	if (battery_status->is_charging) {
-		gpm_debug ("Resetting last_primary_warning to NONE");
+		gpm_debug ("Resetting warning to NONE as charging");
 		manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	}
 
@@ -1327,10 +1328,10 @@ battery_status_changed_primary (GpmManager	      *manager,
 	 * 99 to 100 some laptops report this as charging, some as not-charging.
 	 * This is probably a race. This method should work for both cases. */
 	if (manager->priv->last_primary_percentage_change == 99 &&
-	    battery_status->percentage_charge == 100) {
+	    battery_status->percentage_charge == 100 &&
+	    ! manager->priv->done_notify_fully_charged) {
 		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
 						     GPM_PREF_NOTIFY_BATTCHARGED, NULL);
-
 		if (show_notify) {
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 					      GPM_NOTIFY_TIMEOUT_SHORT,
@@ -1338,6 +1339,13 @@ battery_status_changed_primary (GpmManager	      *manager,
 					      NULL,
 					      _("Your battery is now fully charged"));
 		}
+		manager->priv->done_notify_fully_charged = TRUE;
+	}
+	/* We only re-enable the fully charged notification when the battery
+	   drops down to 95% as some batteries charge to 100% and then fluctuate
+	   from ~98% to 100%. See #338281 for details */
+	if (manager->priv->last_primary_percentage_change < 95) {
+		manager->priv->done_notify_fully_charged = FALSE;
 	}
 
 	manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
@@ -1780,6 +1788,14 @@ hal_battery_removed_cb (GpmHalMonitor *monitor,
 	tray_icon_update (manager);
 }
 
+static void
+tray_icon_destroyed (GtkObject *object, gpointer user_data)
+{
+	GpmManager *manager = user_data;
+
+	manager->priv->tray_icon = gpm_tray_icon_new ();
+}
+
 static void
 gpm_manager_init (GpmManager *manager)
 {
@@ -1836,6 +1852,8 @@ gpm_manager_init (GpmManager *manager)
 
 	gpm_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
+	g_signal_connect (G_OBJECT (manager->priv->tray_icon), "destroy",
+			  G_CALLBACK (tray_icon_destroyed), manager);
 
 #if ACTIONS_MENU_ENABLED
 	gpm_manager_can_suspend (manager, &enabled, NULL);
@@ -1870,6 +1888,7 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->last_pda_warning = GPM_WARNING_NONE;
 	manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	manager->priv->last_primary_percentage_change = 0;
+	manager->priv->done_notify_fully_charged = FALSE;
 
 	/* We don't want to be notified on coldplug if we are on battery power
 	   this should fix #332322 */
